package e_voting;

public class CountingServer {
	private Result result;
	private int numberOfCandidates;
	private int[] ballots;
	/*! result ballots numberOfCandidates | ballots ; !*/

	public CountingServer(){
		result = null;
		numberOfCandidates=0;
		ballots=null;
	}
	public CountingServer(int numberOfCandidates, int numberOfVoters){
		this.numberOfCandidates = numberOfCandidates;
		this.ballots = new int[numberOfVoters];
	}
	/*@escapes ballots;@*/
	public void addBallot(int idx, int vote){
		ballots[idx] = vote;
	}
		
	/*@normal_behavior
	 @requires numberOfCandidates>0 && ballots.length>0 ;
	 @ escapes
	@  (\seq_def int i; 0; numberOfCandidates;
	@   (\num_of int j; 0 <= j && j < ballots.length; i==ballots[j]));
	 	@*/
	public void countBallots(){
		result = new Result(numberOfCandidates);		
		/*@loop_invariant 
       i>=0 && 
       i<= ballots.length && 
       (\forall int k; k>=0 && k< numberOfCandidates; 
         result.bulletin[k] == (\sum int j; 0 <= j && j < i; (k==ballots[j] ? 1 : 0)))  ; 
      @assignable result.bulletin[*],i;
      @decreases ballots.length-i;
      @*/
		for(int i=0; i < ballots.length; i++)
			if(ballots[i]>=0 && ballots[i]<numberOfCandidates)
				result.bulletin[ballots[i]]++;	
		clearBallots();		
	}
	
	public Result getResult(){
		clearBallots();
		return result;
	}

	private void clearBallots(){
		ballots = new int[0];
	}
}
